import { AggregateClass } from '../types/aggregate.type';
import { AnyDomainEvent } from './domain-event';
import { Entity, GetEntityProps } from './entity';
import 'reflect-metadata';
export declare abstract class Aggregate<P> extends Entity<P> {
    protected _originalVersion: number;
    protected _events: AnyDomainEvent[];
    protected _loaded: boolean;
    constructor(type: string, id: string, props: P, originalVersion: number, loaded: boolean);
    static isAggregate(obj: object): obj is AnyAggregate;
    static initAggregate<T extends AnyAggregate>(this: AggregateClass<T>, props: GetEntityProps<T>, id?: string): T;
    static loadAggregate<T extends AnyAggregate>(this: AggregateClass<T>, id: string, props: GetEntityProps<T>, version: number): T;
    hasUnpublishedEvents(): boolean;
    protected recordEvent(event: AnyDomainEvent): void;
    clearEvents(): void;
    isNew(): boolean;
    get events(): AnyDomainEvent[];
    get originalVersion(): number;
    get version(): number;
    get loaded(): boolean;
}
export type AnyAggregate = Aggregate<any>;
export type AggregateConstructorParamsWithProps<P> = ConstructorParameters<typeof Aggregate<P>>;
export type AggregateConstructorParams<T extends AnyAggregate> = AggregateConstructorParamsWithProps<GetEntityProps<T>>;
