"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Aggregate = void 0;
const decorators_1 = require("src/decorators");
const uuid_1 = require("uuid");
const entity_1 = require("./entity");
require("reflect-metadata");
class Aggregate extends entity_1.Entity {
    constructor(type, id, props, originalVersion, loaded) {
        super(type, id, props);
        if (originalVersion < 0)
            throw new Error('Version must be set with non-negative number');
        this._originalVersion = originalVersion;
        this._events = [];
        this._loaded = loaded;
        if (this.isNew()) {
            this.mark();
        }
    }
    static isAggregate(obj) {
        return obj instanceof Aggregate;
    }
    static initAggregate(props, id = (0, uuid_1.v4)()) {
        const aggregateType = Reflect.getMetadata(decorators_1.AGGREGATE_TYPE, this);
        return new this(aggregateType, id, props, 0, false);
    }
    static loadAggregate(id, props, version) {
        const aggregateType = Reflect.getMetadata(decorators_1.AGGREGATE_TYPE, this);
        return new this(aggregateType, id, props, version, true);
    }
    hasUnpublishedEvents() {
        return Boolean(this.events.length);
    }
    recordEvent(event) {
        this._events.push(event);
    }
    clearEvents() {
        this._events = [];
    }
    // mean has no change was saved before
    isNew() {
        return this._originalVersion === 0 && !this._loaded;
    }
    get events() {
        return this._events;
    }
    get originalVersion() {
        return this._originalVersion;
    }
    get version() {
        if (this.hasChanged())
            return this._originalVersion + 1;
        return this._originalVersion;
    }
    get loaded() {
        return this._loaded;
    }
}
exports.Aggregate = Aggregate;
