import { ValueObjectClass } from '../types/value-object.type';
export declare abstract class ValueObject<P> {
    private readonly _type;
    protected readonly _props: Readonly<P>;
    constructor(type: string, props: P);
    static initValueObject<T extends AnyValueObject>(this: ValueObjectClass<T>, props: GetValueObjectProps<T>): T;
    static isValueObject(obj: any): boolean;
    abstract validateProps(props: P): void;
    equalsType(obj: ValueObject<P>): boolean;
    equals(obj: ValueObject<P>): boolean;
    cloneWith(props: Partial<P>): this;
    clone(): this;
    get type(): string;
    get props(): Readonly<P>;
}
export type AnyValueObject = ValueObject<any>;
export type GetValueObjectProps<T extends AnyValueObject> = T extends ValueObject<infer Props> ? Props : any;
export type ValueObjectConstructorParams<T extends AnyValueObject> = ConstructorParameters<typeof ValueObject<GetValueObjectProps<T>>>;
